(ns wanijo.instance.routes
  (:require [compojure.core :refer [defroutes wrap-routes
                                    GET POST DELETE]]
            [ring.util.response :as resp]
            [formulare.core :as form]
            [wanijo.instance
             [view :as view]
             [domain :as domain]
             [forms :as forms-inst]]
            [wanijo.schema
             [domain :as domain-schema]
             [middleware :as middleware-schema]]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.attribute.domain :as domain-attr]))

(defn list! [schema-uuid req]
  (view/list! (domain-schema/find-by-uuid! schema-uuid)
              (domain/find-by-schema! schema-uuid)
              (forms-inst/with-attributes (domain-attr/required! schema-uuid))
              req))

(defn new! [req]
  (let [schema-uuid (get-in req [:params :schema-uuid])
        req-attrs (domain-attr/required! schema-uuid)
        form-def (forms-inst/with-attributes req-attrs)]
    (if (form/valid? form-def req)
      (let [form-data (form/form-data form-def req)
            req-attrs (domain-attr/required! schema-uuid)
            instance (forms-inst/form-data->instance form-data
                                                     req-attrs)]
        (domain/create! schema-uuid
                        instance)
        (resp/redirect (path :instance-list
                             (:params req))))
      (list! schema-uuid req))))

(defn form! [uuid]
  (forms-inst/with-attributes
    (domain-attr/find-by-instance! uuid)))

(defn instance! [uuid]
  (assoc (domain/find-by-uuid! uuid)
         :properties
         (domain/find-properties! uuid)))

(defn show! [uuid req]
  (view/show! (instance! uuid)
              (domain-attr/find-by-instance! uuid)
              req))

(defn edit-form! [uuid req]
  (let [instance (instance! uuid)
        attrs (domain-attr/find-by-instance! uuid)]
    (view/edit! instance
                (form! uuid)
                (forms-inst/instance->form-data instance)
                req)))

(defn edit! [uuid req]
  (let [form-def (form! uuid)
        instance (instance! uuid)]
    (if (form/valid? form-def req)
      (let [form-data (form/form-data form-def req)
            attrs (domain-attr/find-by-instance! uuid)
            form-instance (forms-inst/form-data->instance form-data attrs)
            instance (assoc form-instance :uuid uuid)]
        (domain/edit! instance)
        (resp/redirect (path :instance-show instance)))
      (show! uuid req))))

(defroutes routes
  (wrap-routes (GET (register! :instance-list "/instance/list/:schema-uuid")
                    [schema-uuid :as req]
                    (list! schema-uuid req))
               (middleware-schema/wrap-allowed-to-read
                #(get-in % [:params :schema-uuid])))
  (POST (register! :instance-new "/instance/new") []
        new!)
  (GET (register! :instance-show "/instance/:uuid")
       [uuid :as req]
       (show! uuid req))
  (GET (register! :instance-edit-form "/instance/:uuid/edit")
       [uuid :as req]
       (edit-form! uuid req))
  (POST (register! :instance-edit "/instance/:uuid")
        [uuid :as req]
        (edit! uuid req)))
