(ns wanijo.instance.view
  (:require [hiccup.form :as hform]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [formulare.core :as form]
            [wanijo.instance.domain :as domain]
            [wanijo.framework.view :as view]
            [wanijo.framework.routing :refer [path]]
            [wanijo.framework.time :refer [prettify-dt]]))

(defn list! [schema instances new-form req]
  (view/layout!
   :request req
   :content
   [[:h1 "All Instances of schema "
     [:span.schema-title__name (:name schema)]]
    [:table
     [:thead
      [:tr
       [:th "Name"]
       [:th "Updated"]
       [:th "Created"]]]
     [:tbody
      (for [instance instances]
        [:tr
         [:td
          [:a {:href (path :instance-show instance)}
           (:name instance)]]
         [:td (prettify-dt (:updated_at instance))]
         [:td (prettify-dt (:created_at instance))]])]]
    [:h1 "New Instance"]
    (hform/form-to [:post (path :instance-new)]
                   (form/render-widgets new-form {} req)
                   (hform/hidden-field "schema-uuid"
                                       (:uuid schema))
                   (hform/submit-button "Create!"))]))

(defn show! [instance attrs req]
  (view/layout!
   :request req
   :content
   [[:h1
     (-> instance :schema :name)
     "&nbsp;"
     [:small (:name instance)]]
    [:p
     [:small [:a {:href (path :instance-edit-form instance)}
              "Edit Instance"]]]
    (for [attr attrs
          :let [auuid (:uuid attr)
                prop (first (filter #(= auuid (-> % :attribute :uuid))
                                    (:properties instance)))]]
      (list [:em (:name attr)]
            [:p (:value prop)]))]))

(defn edit! [instance form form-data req]
  (view/layout!
   :request req
   :content
   [[:h1 (:name instance)]
    (hform/form-to [:post (path :instance-edit instance)]
                   (form/render-widgets form form-data req)
                   (hform/submit-button "Edit!"))]))
